<?php
/**
 * DokuWiki farm admin plugin
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Etienne MELEARD <etienne.meleard AT renater.fr>
 */

if(!defined('DOKU_INC')) define('DOKU_INC', realpath(dirname(__FILE__).'/../../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN', DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'admin.php');

require_once(DOKU_PLUGIN.'/farm/dw_farm_animal.class.php');

if(!defined('DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS'))
    define('DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS', DOKU_DEFAULT.'farm_remote_applications.conf');

/**
 * Farm admin plugin
 */
class admin_plugin_farm extends DokuWiki_Admin_Plugin {
    /**
     * Command
     */
    private $cmd = null;
    
    /**
     * Error holder
     */
    private $error = false;
    
    /**
     * Constructor called before admin home/plugin page display
     */
    public function __construct() {
        $this->setupLocale();
    }
    
    /**
     * Get customized menu text
     * 
     * @param string $lang code
     * 
     * @return string
     */
    public function getMenuText($lang) {
        return $this->getLang('menu_'.(DOKU_FARM_IS_FARMER ? 'farmer' : 'animal'));
    }
    
    /**
     * Handle user request
     * 
     * @return mixed
     */
    public function handle() {
        return DOKU_FARM_IS_FARMER ? $this->handle_farmer() : $this->handle_animal();
    }
    
    /**
     * Render page
     * 
     * @return mixed
     */
    public function html() {
        return DOKU_FARM_IS_FARMER ?  $this->html_farmer() : $this->html_animal();
    }
    
    /**
     * Handle farmer request
     */
    private function handle_farmer() {
        $cmd = array_key_exists('command', $_POST) ? key($_POST['command']) : null;
        if(in_array($cmd, array('save_remoteapps'))) $this->cmd = $cmd;
        
        if($this->cmd == 'save_remote_controllers') {
            if(array_key_exists('remote_controllers', $_POST)) {
                $ctn = str_replace("\r", '', trim($_POST['remote_controllers']));
                if($fp = fopen(DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS, 'w')) {
                    fwrite($fp, $ctn);
                    fclose($fp);
                }else $this->error = true;
            }else $this->error = true;
            return;
        }
    }
    
    /**
     * Render farmer admin page
     */
    public function html_farmer() {
        global $conf;
        
        if($this->cmd) msg($this->getLang($this->cmd.'_'.($this->error ? 'error' : 'success')), $this->error ? -1 : 1);
        
        echo $this->locale_xhtml('farmer_page');
        ptln('<div class="level1">');
        $ctn = @file_get_contents(DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS);
        $this->form(array(
            '<textarea name="remote_controllers" rows="15">'.DOKU_LF.formText($ctn).'</textarea>',
            form_button(array('type' => 'submit', 'name' => 'command[save_remote_controllers]', 'value' => $this->getLang('btn_save')))
        ));
        ptln('</div>');
    }
    
    /**
     * Handle animal request
     */
    private function handle_animal() {
        
    }
    
    /**
     * Render animal admin page
     */
    public function html_animal() {
        global $conf;
        
        echo $this->locale_xhtml('animal_page');
        ptln('<div class="level1">');
        $a = new dw_farm_animal(DOKU_FARM_ANIMAL_ID);
        ptln('<ul>');
        foreach($a->getStats() as $t => $s) {
            ptln('<li>'.$this->getLang($t.'_stats'));
            ptln('<ul>');
            foreach($s as $k => $v) {
                if($k != 'cnt') $v = filesize_h($v);
                ptln('<li>'.$this->getLang($k.'_statentry').' : '.$v.'</li>');
            }
            ptln('</ul>');
            ptln('</li>');
        }
        ptln('</ul>');
        ptln('</div>');
    }
    
    /**
     * Build and print a form
     * 
     * @param array $elements
     */
    private function form($elements) {
        global $ID;
        $f = new Doku_Form($this->getPluginName().'__admin', wl($ID, array('do' => 'admin', 'page' => $this->getPluginName()), false, '&'));
        $f->startFieldset('');
        foreach($elements as $e) $f->addElement($e);
        $f->endFieldset();
        echo $f->getForm();
    }
    
    /**
     * Getter
     * 
     * @param string $k key
     * 
     * @return mixed
     */
    public function __get($k) {
        global $conf;
        switch($k) {
            default : return null;
        }
    }
}
