<?php
/**
 * farm action handler
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Etienne MELEARD <etienne.meleard AT renater.fr>
 */

if(!defined('DOKU_INC')) define('DOKU_INC', realpath(dirname(__FILE__).'/../../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN', DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'remote.php');

require_once(DOKU_PLUGIN.'/farm/dw_farm_animal.class.php');

if(!defined('DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS'))
    define('DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS', DOKU_DEFAULT.'farm_remote_applications.conf');

/**
 * DokuWiki farm XML-RPC plugin
 */
class remote_plugin_farm extends DokuWiki_Remote_Plugin {
    /**
     * Remote applications
     */
    private static $remote_applications = null;
    
    /**
     * Current remote application
     */
    private static $remote_application = null;
    
    /**
     * Get available methods
     * 
     * @return array
     */
    public function _getMethods() {
        return array(
            'exists' => array(
                'args' => array('string', 'string', 'string'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'exists(string $remote_application, string $signature, string $name) Test if an animal exists'
            ),
            'getList' => array(
                'args' => array('string', 'string', 'string'),
                'return' => 'array',
                'public' => true,
                'doc' => 'getList(string $remote_application, string $signature, string $filter_host) Get animal list'
            ),
            'getAllStats' => array(
                'args' => array('string', 'string', 'string'),
                'return' => 'struct',
                'public' => true,
                'doc' => 'getAllStats(string $remote_application, string $signature, string $filter_host) Get statistics for all animals'
            ),
            'create' => array(
                'args' => array('string', 'string', 'string', 'string', 'struct'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'create(string $remote_application, string $signature, string $id, string $template, array $customizations) Create a new animal'
            ),
            'edit' => array(
                'args' => array('string', 'string', 'string', 'struct'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'edit(string $remote_application, string $signature, string $id, array $changes) Edit an animal'
            ),
            'editConfig' => array(
                'args' => array('string', 'string', 'string', 'struct', 'string'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'editConfig(string $remote_application, string $signature, string $id, array $changes, string $target) Edit an animal config'
            ),
            'editACL' => array(
                'args' => array('string', 'string', 'string', 'array', 'boolean'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'editACL(string $remote_application, string $signature, string $id, array $acl, bool $override) Add / change acl'
            ),
            'editPage' => array(
                'args' => array('string', 'string', 'string', 'string', 'string', 'boolean'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'editPage(string $remote_application, string $signature, string $id, string $page, string $data) Change page content'
            ),
            'getStats' => array(
                'args' => array('string', 'string', 'string'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'getStats(string $remote_application, string $signature, string $id) Get stats'
            ),
            'delete' => array(
                'args' => array('string', 'string', 'string'),
                'return' => 'boolean',
                'public' => true,
                'doc' => 'delete(string $remote_application, string $signature, string $id) Delete animal'
            ),
        );
    }
    
    /**
     * Verify request signature
     * 
     * @param string $signature received signature
     * @param string $method called method
     * @param string $data_as_string
     */
    public function verifySignature($signature, $method, $data_as_string = '') {
        if(is_null(self::$remote_applications)) {
            self::$remote_applications = array();
            $ctn = preg_replace('`#.+$`m', '', @file_get_contents(DOKU_PLUGIN_FARM_REMOTE_APPLICATIONS)); // Remove comments
            $ctn = preg_replace('`\n(\s*\n)+`', "\n\n", $ctn);
            foreach(explode("\n\n", $ctn) as $block) {
                $b = array('remote_application' => null, 'secret' => null, 'methods' => array());
                foreach(array_map('trim', explode("\n", $block)) as $line) {
                    list($k, $v) = preg_split('`\s+`', $line, 2);
                    if(!array_key_exists($k, $b)) continue;
                    $v = trim($v);
                    if($k == 'methods') $v = array_map('trim', explode(',', $v));
                    $b[$k] = $v;
                }
                if($b['remote_application']) self::$remote_applications[$b['remote_application']] = $b;
            }
        }
        
        if(!array_key_exists($this->remote_application, self::$remote_applications)) throw new RemoteException('Bad signature', 0);
        $dfn = self::$remote_applications[$this->remote_application];
        
        if($signature != hash_hmac('sha1', $data_as_string, $dfn['secret'])) throw new RemoteException('Bad signature', 0);
        
        if(in_array('*', $dfn['methods'])) return;
        if(in_array($method, array('exists'))) return;
        if(!in_array($method, $dfn['methods'])) throw new RemoteException('Not allowed', 1);
    }
    
    /**
     * Order argument(s)
     * 
     * @param mixed $arg
     * 
     * @return mixed
     */
    private function orderArg($arg) {
        if(!is_array($arg)) return $arg;
        
        ksort($arg);
        
        return array_map(array($this, 'orderArg'), $arg);
    }
    
    /**
     * Call handlers wrapper that verifies the signature
     * 
     * @param string $method
     * @param array $args
     */
    public function __call($method, $args) {
        $this->remote_application = array_shift($args);
        $signature = array_shift($args);
        
        $data = array();
        foreach($args as $arg) {
            if(is_bool($arg)) $data[] = $arg ? 1 : 0;
            elseif(is_numeric($arg) || is_string($arg)) $data[] = $arg;
            else $data[] = str_replace('\\/', '/', json_encode($this->orderArg($arg)));
        }
        
        $this->verifySignature($signature, $method, implode('&', $data));
        
        try {
            return call_user_func_array(array($this, 'handle_'.$method), $args);
        } catch(dw_farm_exception $e) {
            throw new RemoteException($e->getMessage(), (int)$e->getCode());
        }
    }
    
    /**
     * Logger
     * 
     * @param string $message
     */
    public function log($message) {
        error_log($message);
    }
    
    /**
     * Test if an animal exists
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * 
     * @return bool
     */
    public function handle_exists($id) {
        $this->log($this->remote_application.' tested if "'.$id.'" animal exists.');
        return dw_farm_animal::exists($id);
    }
    
    /**
     * Get existing wikis list
     * 
     * @param string $filter_host used if DOKU_FARM_MULTIHOSTING, empty means "all hosts", wilcard accepted
     * 
     * @return array
     */
    public function handle_getList($filter_host = null) {
        $this->log($this->remote_application.' requested animal list.');
        return array_keys(dw_farm_animal::getList($filter_host));
    }
    
    /**
     * Get statistics about existing wikis
     * 
     * @param string $filter_host used if DOKU_FARM_MULTIHOSTING, empty means "all hosts", wilcard accepted
     * 
     * @return array
     */
    public function handle_getAllStats($filter_host = null) {
        $this->log($this->remote_application.' requested statistics on all animals.');
        return dw_farm_animal::getAllStats($filter_host);
    }
    
    /**
     * Create a new animal
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * @param string $template <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * @param array $customizations
     * 
     * @return bool
     */ 
    public function handle_create($id, $template, array $customizations) {
        $this->log($this->remote_application.' requested "'.$id.'" animal creation based on "'.$template.'" template with customizations :');
        foreach(explode("\n", print_r($customizations, true)) as $l) $this->log($l);
        dw_farm_animal::create($id, $template, $customizations);
        return true;
    }
    
    /**
     * Apply changes
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * @param array $changes
     * 
     * @return bool
     */
    public function handle_edit($id, array $changes) {
        $this->log($this->remote_application.' requested "'.$id.'" animal edition.');
        $a = new dw_farm_animal($id);
        $a->edit($changes);
        return true;
    }
    
    /**
     * Modify config
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * @param array $config
     * @param string $target config type
     * 
     * @return bool
     */
    public function handle_editConfig($id, array $config, $target = null) {
        $this->log($this->remote_application.' requested "'.$id.'" animal config edition (target : '.$target.').');
        $a = new dw_farm_animal($id);
        $a->editConfig($config, $target);
        return true;
    }
    
    /**
     * Modify ACLs
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * @param array $acl
     * @param bool $override
     * 
     * @return bool
     */
    public function handle_editACL($id, array $acl, $override = false) {
        $this->log($this->remote_application.' requested "'.$id.'" animal ACL edition (override : '.($override ? 'true' : 'false').').');
        $a = new dw_farm_animal($id);
        $a->editACL($acl, $override);
        return true;
    }
    
    /**
     * Edit page source
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * @param mixed $page page path
     * @param mixed $data content
     * @param bool $searchandreplace
     * 
     * @return bool
     */
    public function handle_editPage($id, $page, $data) {
        $this->log($this->remote_application.' requested "'.$id.'" animal "'.$page.'" page edition.');
        $a = new dw_farm_animal($id);
        $a->editPage($page, $data);
        return true;
    }
    
    /**
     * Get statistics about the animal
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * 
     * @return array
     */
    public function handle_getStats($id) {
        $this->log($this->remote_application.' requested "'.$id.'" animal statistics.');
        $a = new dw_farm_animal($id);
        return $a->getStats();
    }
    
    /**
     * Delete the animal
     * 
     * @param string $id <animal_id> or <host>/<animal_id> if DOKU_FARM_MULTIHOSTING
     * 
     * @return bool
     */
    public function handle_delete($id) {
        $this->log($this->remote_application.' requested "'.$id.'" animal deletion.');
        $a = new dw_farm_animal($id);
        $a->delete();
        return true;
    }
}
